/* File:  e__smo.c */
/*

  e__smo.c is a private MATLAB .mex file for computing the fast
  sums associated with command e_smooth() in the EMG Amplitude
  Estimation Toolbox.  This routine is only called from within
  e_smooth(), thus most error checking will already be assumed to
  have been performed by e_smooth().  The calling syntax is:

        e__smo(EMGsmooth, EMGcomp, FixWin, h, Option)

  where FixWin is the integer fixed window length, h is a scalor
  causality offset and Option is an integer (1 ==> 'SumFast',
  2 ==> 'SumFastLong') and the remaining arguments are documented
  in e_smooth().  Fixed length smoothing with a fixed causality is assumed.

  This mex file computes the moving sum (not normalized by the
  appropriate window length) for each output sample.  Variable
  EMGsmooth is included in the argument list to preserve
  memory space (local space within e__smo() need not be allocated
  for it).

  EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

  Copyright (c) Edward A. Clancy, 2007.
  This work is licensed under the Aladdin free public license.
  For copying permissions see license.txt.
  email: ted@wpi.edu

  5 February 2007.
*/

#include "mex.h"

void mexFunction( int nlhs,       mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{

 double *EMGsmooth, *EMGcomp;
 int FixWin, h, Option;
 int i, N; /* Summing index; total number of samples. */
  
 /* Check for proper number of arguments */
 if (nrhs != 5) mexErrMsgTxt("E_SMO.C requires five input arguments.");
 else if (nlhs > 0) mexErrMsgTxt("E_SMO.C requires zero output arguments.");

 /* Get input arguments. */
 EMGsmooth = mxGetPr( prhs[0] );
 N         = mxGetM( prhs[0] );
 EMGcomp   = mxGetPr( prhs[1] );
 FixWin    = (int) mxGetScalar( prhs[2] );
 h         = (int) mxGetScalar( prhs[3] );
 Option    = (int) mxGetScalar( prhs[4] );

 /* Perform fast summing, based on the input option. */
 switch( Option ) {
   case 1: /* 'SumFast'. */
     /* First element. */
     EMGsmooth[0] = 0.0;
     for(i=0; i<=h; i++) EMGsmooth[0] += EMGcomp[i];
     /* Left edge. */
     for(i=1; i<FixWin-h; i++) EMGsmooth[i] = EMGsmooth[i-1] + EMGcomp[i+h];
     /* Main body. */
     for(; i<N-h; i++)
       EMGsmooth[i] = EMGsmooth[i-1] + EMGcomp[i+h] - EMGcomp[i+h-FixWin];
     /* Right edge. */
     for(; i<N; i++) EMGsmooth[i] = EMGsmooth[i-1] - EMGcomp[i+h-FixWin];
     break;
   case 2: /* 'SumFastLong'. */
     mexErrMsgTxt("'SumFastLong' not programmed yet.");
     break;
   default:
     mexErrMsgTxt("Bogus value for 'Option'.");
 }

 return;
}


