/* File:  e__whi.c */
/*

  e__whi.c is a private MATLAB .mex file for rapid computation
  of adaptive whitening filters in the EMG Amplitude Estimation
  Toolbox.  This routine is only called from within e_whiten(),
  thus most error checking will already be assummed to have
  been performed by e_whiten().  The calling syntax is:

       e__whi(EMGwhite, EMGin, Bwhite, Bindex, h)

  where the arguments are documented in e_whiten().


  This mex file computes the adaptive finite impulse response
  (FIR) filter specified by Bwhite, Bindex and h.  Variable
  EMGwhite is included in the argument list (it is assumed to be
  pre-allocated to a vector of zeros of the correct size) to preserve
  memory space (local space within e__whi() need not be allocated
  for it).

  Made a mex file by changing the MATLAB current directory to the
  directory of this file and typing "mex -O e__whi.c".

  EMG Amplitude Estimation Toolbox - Ted Clancy - WPI
  March 30, 1999
*/

#include "mex.h"

void mexFunction( int nlhs,       mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{

 double *Bindex,
        *Bwhite,
        *EMGin,
        *EMGwhite;
 int  Bcol, /* Number of columns (filter length) in Bwhite.     */
      Brow, /* Number of rows (number of filters) in Bwhite.    */
      h;
 long Elen, /* Number of EMG samples (input and output vector). */
      Head, /* Output samples in the leading edge.              */
      i,    /* Sample index.                                    */
      j,    /* Summing index.                                   */
            Tail; /* Output samples in the trainling edge.            */
  
 /* Check for proper number of arguments */
 if (nrhs != 5) mexErrMsgTxt("E__WHI.C requires five input arguments.");
 else if (nlhs > 0) mexErrMsgTxt("E__WHI.C requires zero output arguments.");

 /* Get input arguments. */
 EMGwhite = mxGetPr( prhs[0] );
 Elen     = mxGetN( prhs[0] );
 EMGin    = mxGetPr( prhs[1] );
 Bwhite   = mxGetPr( prhs[2] );
 Brow     = mxGetM( prhs[2] );
 Bcol     = mxGetN( prhs[2] );
 Bindex   = mxGetPr( prhs[3] );
 h        = (int) mxGetScalar( prhs[4] );

 /* Determine sample summing ranges to deal with edges. */
 Head = Bcol - 1 - h;
 Tail = Elen + 1 - h;

 /* Process leading edge. */
 for(i=0; i<Head; i++) {
   for(j=0; j<=i+h; j++) {
     EMGwhite[i] += EMGin[(int)j] * Bwhite[(int)Brow*((int)i+(int)h-(int)j)+(int)(Bindex[(int)i]-1)];
   }
 }

 /* Process body. */
 for(i=Head; i<Tail-1; i++) {
   for(j=0; j<Bcol; j++) {
     EMGwhite[(int)i] += EMGin[(int)i+(int)h-(int)Bcol+1+(int)j] * Bwhite[(int)Brow*((int)Bcol-1-j)+(int)(Bindex[i]-1)];
   }
 }

 /* Process trailing edge. */
 for(i=Tail-1; i<Elen; i++) {
   for(j=0; j<Elen-i+Bcol-h-1; j++) {
     EMGwhite[(int)i] += EMGin[(int)i+(int)h-(int)Bcol+1+(int)j] * Bwhite[(int)Brow*((int)Bcol-1-(int)j)+(int)(Bindex[i]-1)];
   }
 }

 return;
}


