function EMGinfo = e_cal(SandNmat, sCal, NoiseMat, SampFreq, varargin)
%E_CAL Perform a complete calibration for EMG amplitude estimation.
%
% EMGinfo = e_cal(SandNmat, sCal, NoiseMat, SampFreq, ...
%                 'PropertyName', PropertyValue, ...)
%
% 'PropertyName', PropertyValues which can be set by the user:
%
% Causality       Causality of the EMG processing.  Must be set to one of the
%                   values 'Noncausal' or 'Causal'.
% DemodFlag       'On' or 'Off'.  Used to turn off demodulation and
%                   relinearization.  Used when e_amp does
%                   not perform a complete amplitude estimate.
% DemodM          Demodulate/relinearize order.
% HpassFlag       'Filter', 'Off' or 'Offset'.  ('Filter' ==> Filter,
%                   'Offset' ==> Offset only, 'Off' ==> None.)
% HpassOrder      Scalor high pass filter order.
% HpassWn         Scalor high pass filter cut-off (0--1).
% SmoothEdges     Smoothing filter edges option ('MAV' usage).
% SmoothFilt      Smoothing filter usage specification.
%                   'MAV', 'butter', 'ellip', 'cheby1' or 'cheby2'.
% SmoothFixWin    Fixed smoothing window for the smoothing stage ('MAV' usage).
% SmoothOrder     Smoothing filter order (linear filter usage).
% SmoothSum       Smoothing filter summing option ('MAV' usage).
% SmoothWn        Smoothing filter cut-off (linear filter usage).
% UncorrFlag      'On' ==> uncorrelate, 'Off' ==> none, 
%                   'GainOnly' ==> 'GainOnly' option in e_cal_sp().
% WhiteEdges      Whitening filter 'Edges' option.
% WhiteFlag       'On' or 'Off'.  Used to turn off whitening.
% WhiteMaxGain    Whitening calibration 'MaxGain' option.
% WhiteNfft       Whitening calibration 'Nfft' option.
% WhiteOrder      Whitening calibration order.
% White_sSpec     Whitening sSpec vector.
% WhiteSmFilt     Specifies the "SmoothFilt" for the first-pass amplitude
%                   estimate used in whitening.  Must be specified as
%                   "SmoothFilt".
% WhiteSmFixWin   Specifies the "SmoothFixWin" for the first-pass amplitude
%                   estimate used in whitening.  Must be specified as
%                   "SmoothFixWin".
% WhiteSmOrder    Specifies the "SmoothOrder" for the first-pass amplitude
%                   estimate used in whitening.  Must be specified as
%                   "SmoothOrder".
% WhiteSmWn       Specifies the "SmoothWn" for the first-pass amplitude
%                   estimate used in whitening.  Must be specified as
%                   "SmoothWn".
% WhiteSum        Whitening filter 'Sum' option.
%
%
%
% 'PropertyName', PropertyValues which are set internally:
%
% Channels        Scalor number of EMG channels.
% HpassOffset     Vector offsets for high pass filter, 'Offset' option.
%                   One offset per EMG channel.
% sCal            MVE level corresponding to SandNmat (scalor).
%                   (Typically 0.5 for 50% MVC.)
% UncorrHspace    Hspace matrix set by e_cal_sp().
% WhiteB          Cell array of matrices of adaptive whitening filter
%                   coefficients.  One cell per channel.
% White_sCalVec   Cell array of vectors of adaptive whitening calibration
%                   vectors.  One cell per channel.
% 
%
% EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

% Copyright (c) Edward A. Clancy, 2007.
% This work is licensed under the Aladdin free public license.
% For copying permissions see license.txt.
% email: ted@wpi.edu

% 5 February 2007.

%%%%%%%%%%%%%%%%%%%%%%%%% Check Command Line Length (Only). %%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4, error(['Bogus argument count (' int2str(nargin) '), expected 4+']); end

ArgsIn = length(varargin);
if floor(ArgsIn/2)*2 ~= ArgsIn, error('Expected even number of optional arguments.'); end

%%%%%%%%%%%%%%%%%%%% Process Command Line Options. %%%%%%%%%%%%%%%%%%%

EMGinfo = [];  % Initialize.

for i = 1:2:ArgsIn
  switch varargin{i}
      
    case 'Causality'
      if isfield(EMGinfo, 'Causality'), error('Repeat "Causality" specification.'); end
      EMGinfo.Causality = varargin{i+1};
      if strcmp(varargin{i+1}, 'Noncausal')==0 & strcmp(varargin{i+1}, 'Causal')==0
        error(['Bogus PropertyName (' varargin{i+1} ') for "Causality".']);
      end

    % Noise Rejection/Filtering.
    case 'HpassFlag'
      if isfield(EMGinfo, 'HpassFlag'), error('Repeat "HpassFlag" specification.'); end
      EMGinfo.HpassFlag = varargin{i+1};
    case 'HpassOrder'
      if isfield(EMGinfo, 'HpassOrder'), error('Repeat "HpassOrder" specification.'); end
      EMGinfo.HpassOrder = varargin{i+1};
    case 'HpassWn'
      if isfield(EMGinfo, 'HpassWn'), error('Repeat "HpassWn" specification.'); end
      EMGinfo.HpassWn = varargin{i+1};

    % Whitening.
    case 'WhiteFlag'
      if isfield(EMGinfo, 'WhiteFlag'), error('Repeat "WhiteFlag" specification.'); end
      EMGinfo.WhiteFlag = varargin{i+1};
    case 'WhiteEdges'
      if isfield(EMGinfo, 'WhiteEdges'), error('Repeat "WhiteEdges" specification.'); end
      EMGinfo.WhiteEdges = varargin{i+1};
    case 'WhiteMaxGain'
      if isfield(EMGinfo, 'WhiteMaxGain'), error('Repeat "WhiteMaxGain" specification.'); end
      EMGinfo.WhiteMaxGain = varargin{i+1};
    case 'WhiteNfft'
      if isfield(EMGinfo, 'WhiteNfft'), error('Repeat "WhiteNfft" specification.'); end
      EMGinfo.WhiteNfft = varargin{i+1};
    case 'WhiteOrder'
      if isfield(EMGinfo, 'WhiteOrder'), error('Repeat "WhiteOrder" specification.'); end
      EMGinfo.WhiteOrder = varargin{i+1};
    case 'White_sSpec'
      if isfield(EMGinfo, 'White_sSpec'), error('Repeat "White_sSpec" specification.'); end
      EMGinfo.White_sSpec = varargin{i+1};
    case 'WhiteSmFilt'
      if isfield(EMGinfo, 'WhiteSmFilt'), error('Repeat "WhiteSmFilt" specification.'); end
      EMGinfo.WhiteSmFilt = varargin{i+1};
    case 'WhiteSmFixWin'
      if isfield(EMGinfo, 'WhiteSmFixWin'), error('Repeat "WhiteSmFixWin" specification.'); end
      EMGinfo.WhiteSmFixWin = varargin{i+1};
    case 'WhiteSmOrder'
      if isfield(EMGinfo, 'WhiteSmOrder'), error('Repeat "WhiteSmOrder" specification.'); end
      EMGinfo.WhiteSmOrder = varargin{i+1};
    case 'WhiteSmWn'
      if isfield(EMGinfo, 'WhiteSmWn'), error('Repeat "WhiteSmWn" specification.'); end
      EMGinfo.WhiteSmWn = varargin{i+1};
    case 'WhiteSum'
      if isfield(EMGinfo, 'WhiteSum'), error('Repeat "WhiteSum" specification.'); end
      EMGinfo.WhiteSum = varargin{i+1};

    % Multiple-Channel Combination (Including Gain Scaling).
    case 'UncorrFlag'
      if isfield(EMGinfo, 'UncorrFlag'), error('Repeat "UncorrFlag" specification.'); end
      EMGinfo.UncorrFlag = varargin{i+1};

    % Demodulate and relinearize.
    case 'DemodFlag'
      if isfield(EMGinfo, 'DemodFlag'), error('Repeat "DemodFlag" specification.'); end
      EMGinfo.DemodFlag = varargin{i+1};
    case 'DemodM'
      if isfield(EMGinfo, 'DemodM'), error('Repeat "DemodM" specification.'); end
      EMGinfo.DemodM = varargin{i+1};

    % Smooth.
    case 'SmoothEdges'
      if isfield(EMGinfo, 'SmoothEdges'), error('Repeat "SmoothEdges" specification.'); end
      if isfield(EMGinfo, 'SmoothFilt') & strcmp(EMGinfo.SmoothFilt, 'MAV')==0
        error('"SmoothFilt" must be "MAV" if "SmoothEdges" is specified.');
      end
      if isfield(EMGinfo, 'SmoothOrder'), error('Can not specify "SmoothOrder" and "SmoothEdges".'); end
      if isfield(EMGinfo, 'SmoothWn'), error('Can not specify "SmoothWn" and "SmoothEdges".'); end
      EMGinfo.SmoothEdges = varargin{i+1};
    case 'SmoothFilt'
      if isfield(EMGinfo, 'SmoothFilt'), error('Repeat "SmoothFilt" specification.'); end
      switch varargin{i+1}
        case 'MAV'
          if isfield(EMGinfo, 'SmoothOrder'), error('Can not specify "SmoothOrder" when "SmoothFilt" = "MAV".'); end
          if isfield(EMGinfo, 'SmoothWn'), error('Can not specify "SmoothWn" when "SmoothFilt" = "MAV".'); end
        otherwise  % Assume linear filter specification.
          if isfield(EMGinfo, 'SmoothEdges'), error('Can not specify "SmoothEdges" when "SmoothFilt" ~= "MAV".'); end
          if isfield(EMGinfo, 'SmoothSum'), error('Can not specify "SmoothSum" when "SmoothFilt" ~= "MAV".'); end
      end
      EMGinfo.SmoothFilt = varargin{i+1};
    case 'SmoothFixWin'
      if isfield(EMGinfo, 'SmoothFixWin'), error('Repeat "SmoothFixWin" specification.'); end
      if isfield(EMGinfo, 'SmoothFilt') & strcmp(EMGinfo.SmoothFilt, 'MAV')==0
        error('"SmoothFilt" must be "MAV" if "SmoothFixWin" is specified.');
      end
      if isfield(EMGinfo, 'SmoothOrder'), error('Can not specify "SmoothOrder" and "SmoothFixWin".'); end
      if isfield(EMGinfo, 'SmoothWn'), error('Can not specify "SmoothWn" and "SmoothFixWin".'); end
      EMGinfo.SmoothFixWin = varargin{i+1};
    case 'SmoothSum'
      if isfield(EMGinfo, 'SmoothSum'), error('Repeat "SmoothSum" specification.'); end
      if isfield(EMGinfo, 'SmoothFilt') & strcmp(EMGinfo.SmoothFilt, 'MAV')==0
        error('"SmoothFilt" must be "MAV" if "SmoothSum" is specified.');
      end
      if isfield(EMGinfo, 'SmoothOrder'), error('Can not specify "SmoothOrder" and "SmoothSum".'); end
      if isfield(EMGinfo, 'SmoothWn'), error('Can not specify "SmoothWn" and "SmoothSum".'); end
      EMGinfo.SmoothSum = varargin{i+1};
    case 'SmoothOrder'
      if isfield(EMGinfo, 'SmoothOrder'), error('Repeat "SmoothOrder" specification.'); end
      if isfield(EMGinfo, 'SmoothFilt') & strcmp(EMGinfo.SmoothFilt, 'MAV')==1
        error('"SmoothFilt" must NOT be "MAV" if "SmoothOrder" is specified.');
      end
      if isfield(EMGinfo, 'SmoothEdges'), error('Can not specify "SmoothEdges" and "SmoothOrder".'); end
      if isfield(EMGinfo, 'SmoothSum'), error('Can not specify "SmoothSum" and "SmoothOrder".'); end
      EMGinfo.SmoothOrder = varargin{i+1};
    case 'SmoothWn'
      if isfield(EMGinfo, 'SmoothWn'), error('Repeat "SmoothWn" specification.'); end
      if isfield(EMGinfo, 'SmoothFilt') & strcmp(EMGinfo.SmoothFilt, 'MAV')==1
        error('"SmoothFilt" must NOT be "MAV" if "SmoothWn" is specified.');
      end
      if isfield(EMGinfo, 'SmoothEdges'), error('Can not specify "SmoothEdges" and "SmoothWn".'); end
      if isfield(EMGinfo, 'SmoothSum'), error('Can not specify "SmoothSum" and "SmoothWn".'); end
      EMGinfo.SmoothWn = varargin{i+1};

    otherwise, error(['Bogus PropertyName "' varargin{i} '".'])

  end  % switch
end  % for

%%%%%%%%%%%%%% Check Required Command Line Arguments. %%%%%%%%%%%%%%

% Set no errors (yet) if an argument is null.

[SNrow, SNcol] = size(SandNmat);
if SNrow==SNcol & SNrow~=0, error('Argument 1 must not be a square matrix.'); end
% Coerce one EMG channel per COLUMN.
if SNcol>SNrow, SandNmat = SandNmat'; [SNrow, SNcol] = size(SandNmat); end
EMGinfo.Channels = min( size(SandNmat) );

if sum(size(sCal))>2, error('Argument 2 must be a scalor.'); end
EMGinfo.sCal = sCal;

[Nrow,  Ncol] = size(NoiseMat);
if Nrow==Ncol & Nrow~=0, error('Argument 3 must not be a square matrix.'); end
% Coerce one EMG channel per COLUMN.
if Ncol>Nrow, NoiseMat = NoiseMat'; [Nrow, Ncol] = size(NoiseMat); end
if SNcol~=Ncol, error('Args 1 and 3 must have same number of EMG channels.'); end

if ischar(SampFreq)==1, error('Argument 4 must be numeric.'); end
if length(SampFreq)>1, error('Argument 4 must be a scalor.'); end
if SampFreq<=0, error('Argument 4 must be > 0.'); end
EMGinfo.SampFreq = SampFreq;

%%%%%%%%%%%%%%%%%%%%%%%%%%%% Set Global Defaults. %%%%%%%%%%%%%%%%%%%%%%%%%

if ~isfield(EMGinfo, 'Causality'), EMGinfo.Causality = 'Noncausal'; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%% Calibrate. %%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Demodulation and relinearization.
%   Set defaults now, so available for first-pass whitening, etc.
if ~isfield(EMGinfo, 'DemodFlag'), EMGinfo.DemodFlag = 'On'; end
if ~isfield(EMGinfo, 'DemodM'),    EMGinfo.DemodM    = []; end


% Smoothing.
%   Set defaults now, so available for first-pass whitening, etc.
if ~isfield(EMGinfo, 'SmoothFilt'),   EMGinfo.SmoothFilt   = 'MAV'; end
if ~isfield(EMGinfo, 'SmoothEdges'),  EMGinfo.SmoothEdges  = []; end
if ~isfield(EMGinfo, 'SmoothFixWin'), EMGinfo.SmoothFixWin = round(EMGinfo.SampFreq./10); end
if ~isfield(EMGinfo, 'SmoothOrder'),  EMGinfo.SmoothOrder  = 5; end
if ~isfield(EMGinfo, 'SmoothSum'),    EMGinfo.SmoothSum    = []; end
if ~isfield(EMGinfo, 'SmoothWn'),     EMGinfo.SmoothWn     = (5*2)./EMGinfo.SampFreq; end


% Calibration for noise rejection/filtering.
%   Set defaults.
if ~isfield(EMGinfo, 'HpassFlag'),  EMGinfo.HpassFlag  = 'Filter'; end
if ~isfield(EMGinfo, 'HpassOrder'), EMGinfo.HpassOrder = []; end
if ~isfield(EMGinfo, 'HpassWn'),    EMGinfo.HpassWn    = (15*2)./EMGinfo.SampFreq; end
%   Calibration only needed if HpassFlag = 'Offset'.
EMGinfo.HpassOffset = [];
if strcmp(EMGinfo.HpassFlag,'Offset')==1
  if isempty(NoiseMat), error('Arg 3 can not be null if "HpassFlag" = "Offset".'); end
  EMGinfo.HpassOffset = mean(NoiseMat);
end


% Calibration for whitening.
%   Set defaults.
if ~isfield(EMGinfo, 'WhiteFlag'),     EMGinfo.WhiteFlag     = 'On'; end
if ~isfield(EMGinfo, 'WhiteEdges'),    EMGinfo.WhiteEdges    = []; end
if ~isfield(EMGinfo, 'WhiteMaxGain'),  EMGinfo.WhiteMaxGain  = []; end
if ~isfield(EMGinfo, 'WhiteNfft'),     EMGinfo.WhiteNfft     = []; end
if ~isfield(EMGinfo, 'WhiteOrder'),    EMGinfo.WhiteOrder    = []; end
if ~isfield(EMGinfo, 'White_sSpec'),   EMGinfo.White_sSpec   = []; end
if ~isfield(EMGinfo, 'WhiteSmFilt'),   EMGinfo.WhiteSmFilt   = EMGinfo.SmoothFilt; end
if ~isfield(EMGinfo, 'WhiteSmFixWin'), EMGinfo.WhiteSmFixWin = EMGinfo.SmoothFixWin; end
if ~isfield(EMGinfo, 'WhiteSmOrder'),  EMGinfo.WhiteSmOrder  = EMGinfo.SmoothOrder; end
if ~isfield(EMGinfo, 'WhiteSmWn'),     EMGinfo.WhiteSmWn     = EMGinfo.SmoothWn; end
if ~isfield(EMGinfo, 'WhiteSum'),      EMGinfo.WhiteSum      = []; end
%   Calibrate whitening filters, if desired.
if strcmp(EMGinfo.WhiteFlag, 'On') == 1
  % Error check required arguments.
  if isempty(SandNmat), error('Arg1 can not be null if "WhiteFlag" = "On".'); end
  if isempty(sCal),     error('Arg2 can not be null if "WhiteFlag" = "On".'); end
  if isempty(NoiseMat), error('Arg3 can not be null if "WhiteFlag" = "On".'); end
  % Noise reject/filter the calibration data before whitening calibration.
  SandN2 = e_amp_nj(SandNmat, EMGinfo);
  Noise2 = e_amp_nj(NoiseMat, EMGinfo);
  % Create whitening filters.
  for i = 1:EMGinfo.Channels
    [EMGinfo.WhiteB{i}, EMGinfo.White_sCalVec{i}] = ...
         e_cal_wh(SandN2(:,i), sCal, Noise2(:,i), EMGinfo.WhiteOrder, EMGinfo.White_sSpec, ...
              'Nfft', EMGinfo.WhiteNfft, 'MaxGain', EMGinfo.WhiteMaxGain);
  end
end


% Calibration for multiple-channel combination (including gain scaling).
%   Set defaults.
if ~isfield(EMGinfo, 'UncorrFlag'), EMGinfo.UncorrFlag = 'GainOnly'; end
%   Calibrate, if desired.
switch EMGinfo.UncorrFlag
  case {'GainOnly', 'On'}
    % Error check required arguments.
    if isempty(SandNmat), error('Arg1 can not be null if "WhiteFlag" = "On".'); end
    if isempty(sCal),     error('Arg2 can not be null if "WhiteFlag" = "On".'); end
    % Noise reject/filter before combination.
    SandN2 = e_amp_nj(SandNmat, EMGinfo);
    % Create spatial uncorrelation matrix.
    if strcmp(EMGinfo.UncorrFlag, 'GainOnly')==1
      % NOTE: Need not whiten first, since whitening does not change signal variance.
      EMGinfo.UncorrHspace = e_cal_sp(SandN2, sCal, 'GainOnly');
    else
      % Must whiten first, since whitening changes correlation (without changing
      %   variance).  But, whitening requires a first-pass estimation, which
      %   requires an Hspace.  So, bootstrap using 'GainOnly'.
      EMGinfo.UncorrHspace = e_cal_sp(SandN2, sCal, 'GainOnly');
      EMGinfo.UncorrHspace = EMGinfo.UncorrHspace * ...
        sCal / meanN0( e_amp(SandNmat, EMGinfo) );  % Adjust gain for demodulator M.
      SandN3 = e_amp_wh(SandN2,   EMGinfo);
      EMGinfo.UncorrHspace = e_cal_sp(SandN3, sCal);  % Actual Hspace estimate.
      clear SandN3;
    end
    % Adjust gain for demodulator M.  (Avoid NaN and Zero head and tail, if present.)
    EMGinfo.UncorrHspace = EMGinfo.UncorrHspace * ...
      sCal / meanN0( e_amp(SandNmat, EMGinfo) );
  case 'Off'
    EMGinfo.UncorrHspace = [];
    if EMGinfo.Channels>1
      error(['EMG channels > 1 (' int2str(EMGinfo.Channels) '), but "UncorrFlag" = "Off".']);
    end
  otherwise, error(['Bogus PropertyName (' EMGinfo.UncorrFlag ') for "UncorrFlag".']);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% meanN0() %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Output = meanN0(Input)
%MEANN0 Find mean of non-NaN, non-zero entries.

IndexZero =  find(Input==0);
IndexNaN  =  find(isnan(Input)==1);

IndexSum = find(isnan(Input)==0);

GrandSum = sum( Input(IndexSum) );

Output = GrandSum ./ ( length(IndexSum) - length(IndexZero) );
