function Hspace = e_cal_sp(EMGin, sCal, GainOnly)
%E_CAL_SP Calibrate a spatial uncorrelation filter.
%
% Hspace = e_cal_sp(EMGin, sCal [, 'GainOnly'])
%
% EMGin: Matrix of L vectors, each vector being the N time-series data
%        samples from one whitened EMG signal.  The larger of the two
%        matrix dimensions is assigned as the sample index, the smaller
%        the channel index; ties produce an error.
% sCal:  MVE level corresponding to EMGin (scalor).
%        (Typically 0.5 for 50% MVC.)
% 'GainOnly': Optional literal string argument.  If supplied, then
%        calibration is based only on each channel's autocorrelation,
%        not any channel cross-correlations.  Thus, each channel is
%        normalized to the value sCal.
%
% Hspace: Square spatial uncorrelation matrix.  Each ROW holds the
%        coefficients used to produce one output channel when spatial
%        uncorrelation is applied.
%
% EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

% Copyright (c) Edward A. Clancy, 2007.
% This work is licensed under the Aladdin free public license.
% For copying permissions see license.txt.
% email: ted@wpi.edu

% 5 February 2007.

% L: Number of EMG channels.

%%%%%%%%%%%%%%%%%%%%% Process Command Arguments %%%%%%%%%%%%%%%%%%%%%

% Check number of input arguments
if nargin<2 | nargin>3, error(['Expected 2-3 arguments, found ' int2str(nargin) '.']); end

% Check EMGin
if ischar(EMGin), error('Argument 1 must be numeric.'); end
if ndims(EMGin)>2, error('Argument 1 must be a matrix.'); end
[Erow, Ecol] = size(EMGin);
if Erow==Ecol, error('Argument 1 can not be a square matix.'); end
if Erow>Ecol, L = Ecol; else L = Erow; end

% Check sCal.
if isstr(sCal), error('Second argument "sCal" not numeric.'); end
if length(sCal)>1, error('Second argument "sCal" not a scalor.'); end
if sCal<=0.0, error('Second argument "sCal" must be non-negative.'); end

% Check GainOnly.
if nargin==3
  if strcmp(GainOnly, 'GainOnly')==0, error('Optional arg 3 must be "GainOnly".'); end
   GainOnly = 1;
else
  GainOnly = 0;  % Default.
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Compute %%%%%%%%%%%%%%%%%%%%%%%%%

% Compute covariance matrix.
if Erow>Ecol, Kmatrix = cov(EMGin); else Kmatrix = cov(EMGin'); end

% Find eigenvalues and eigenvectors of Kmatrix.
[V, D] = eig(Kmatrix);
for i=1:L  % If any eigenvalues are negative, move sign to vector.
  if D(i,i)<0; D(i,i) = -D(i,i); V(:,i) = -V(:,i); end
end

% Form Hspace.
Hspace = zeros(L,L);
if GainOnly==0
  for i = 1:L, Hspace(i,i) = 1 ./ sqrt( D(i,i) ); end
  Hspace = Hspace * (V');
else
  for i = 1:L, Hspace(i,i) = 1 ./ sqrt( Kmatrix(i,i) ); end
end

% Scale to MVE level of calibration sample.
for i = 1:L,, Hspace(i,:) = Hspace(i,:) * sCal; end
