function [B, sCalVec] = e_cal_wh(SandN, sCal, Noise, varargin)
%E_CAL_WH Design/calibrate adaptive and fixed whitening filters.
%
% [B, sCalVec] = e_cal_wh(SandN, sCal, Noise[, Order][, sSpec][, Option-Pairs])
%
% Design a set of FIR adaptive whitening filters based on the three-
% stage cascade technique of Clancy and Farry (In Preparation).
%
% SandN: Calibration sample of signal plus noise (vector).
% sCal:  MVE level corresponding to SandN (scalor).
%        (Typically 0.5 for 50% MVC.)
% Noise: Calibration sample of noise (vector).
%
% Options ---
% Order: Order of the combined filter (# coefficients - 1) (scalor).
%        (Default = 60.)
% sSpec: EMG amplitude level specifier (same scale as sCal) (vector).
%        (Default specifies 100 filters from just above 0 to 2*sCal.)
%
% Option-Pairs ---
% 'Nfft', NNfft:  FFT length (power of 2) for spectral estimates (scalor).
% 'MaxGain', MMaxGain: Maximum design gain at any frequency (scalor).
%
% B: Each row contains the moving average coefficients of the
%    complete adaptive whitener corresponding to the EMG amplitude
%    found in the corresponding element of sCalVec.
% sCalVec: Equally spaced vector of EMG amplitudes specifying the amplitudes
%    for which the filters in B are calibrated.
%
%   EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

% Copyright (c) Edward A. Clancy, 2007.
% This work is licensed under the Aladdin free public license.
% For copying permissions see license.txt.
% email: ted@wpi.edu

% 5 February 2007.

%%%%%%%%%%%%%%%%%%%%% Process Command Arguments %%%%%%%%%%%%%%%%%%%%%

% Check number of input/output arguments
ArgsIn  = nargin;
ArgsVar = length(varargin);
if (ArgsIn < 3  |  ArgsIn > 9)
  error(['Bogus number of arguments (' int2str(ArgsIn) '), expected 3-9.']);
end

% Look for option-pairs.
NNfft = 256;      % Default FFT length (power of 2).
MMaxGain = 100;   % Default maximum fixed whitening gain.
FlagNfft = 0;     % 0 denotes option not yet used: 1==> used.
FlagMax  = 0;     % 0 denotes option not yet used: 1==> used.
NotDone = 1;
while (ArgsVar >= 2)
  if isstr(varargin{ArgsVar-1})
    switch varargin{ArgsVar-1}
      case 'Nfft'
        if (FlagNfft > 0 ), error('Repeated "Nfft" selection.'); end
        FlagNfft = 1;
        if isempty(varargin{ArgsVar})==0, NNfft = varargin{ArgsVar}; end
        if length(NNfft)>1, error('NNfft not a scalor.'); end
        if (2^nextpow2(NNfft))~=NNfft, error('NNfft not a power of 2.'); end
      case 'MaxGain'
        if (FlagMax > 0 ), error('Repeated "MaxGain" selection.'); end
        FlagMax = 1;
        if isempty(varargin{ArgsVar})==0, MMaxGain = varargin{ArgsVar}; end
        if length(MMaxGain)>1, error('MMaxGain not a scalor.'); end
        if MMaxGain <= 0.0, error('MMaxGain must be non-negative.'); end
      otherwise
        error(['Bogus option "' varargin{ArgsVar-1} '".']);
    end
  else
    break;
  end
  ArgsIn  = ArgsIn  - 2;
  ArgsVar = ArgsVar - 2;
end

% Required arguments and regular optional arguments.
if (ArgsIn < 3  |  ArgsIn > 5)
  error(['Bogus number of non-option-pair arguments (' int2str(ArgsIn) '), expected 3-5.']);
end

if isstr(SandN), error('First argument "SandN" not numeric.'); end
SizeRes = size(SandN);
if ( length(SizeRes)>2 | (SizeRes(1)~=1 & SizeRes(2)~=1) )
  error('First argument "SandN" not a vector.');
end
if (length(SandN)<2*NNfft), error('First argument length too short.'); end

if isstr(sCal), error('Second argument "sCal" not numeric.'); end
if length(sCal)>1, error('Second argument "sCal" not a scalor.'); end
if sCal<=0.0, error('Second argument "sCal" must be non-negative.'); end

if isstr(Noise), error('Third argument "Noise" not numeric.'); end
SizeRes = size(Noise);
if ( length(SizeRes)>2 | (SizeRes(1)~=1 & SizeRes(2)~=1) )
  error('Third argument "Noise" not a vector.');
end
if (length(Noise)<2*NNfft), error('Third argument length too short.'); end

Order = 60;  % Default.
if ArgsVar > 0
  if isempty(varargin{1})==0, Order = varargin{1}; end
  if isstr(Order), error('Fourth argument "Order" not numeric.'); end
  if length(Order)>1, error('Fourth argument "Order" not a scalor.'); end
  if Order<=0, error('Fourth argument "Order" must be non-negative.'); end
end

sCalVec = 2*sCal/100 : 2*sCal/100 : 2*sCal;  % Default.
if length(sCalVec)~=100, error('Internal "sCalVec" length error.'); end
if ArgsVar > 1  &  isempty(varargin{2}) == 0
  sSpec = varargin{2};
  if isstr(sSpec), error('Fifth argument "sSpec" not numeric.'); end
  SizeRes = size(sSpec);
  if ( length(SizeRes)>2 | (SizeRes(1)~=1 & SizeRes(2)~=1) )
    error('Fifth argument "sSpec" not a vector.');
  end
  if length(sSpec)~=3, error('Fifth argument "sSpec" not length 3.'); end
  if sSpec(1)<=0.0, error('sSpec(1) less than or equal to zero.'); end
  if sSpec(2)<=0.0, error('sSpec(2) less than or equal to zero.'); end
  if sSpec(3)<sSpec(1), error('sSpec(3) less than sSpec(1).'); end
  sCalVec = sSpec(1) : sSpec(2) : sSpec(3);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Compute %%%%%%%%%%%%%%%%%%%%%%%%%

% Compute magnitude response of the fixed whitener.
%   Estimate PSD's of SandN and Noise.  Also get frequency bins.
[Svv, f] = psd(Noise, NNfft, 2, NNfft, round(0.75*NNfft));
Smm_sCal = psd(SandN, NNfft, 2, NNfft, round(0.75*NNfft));
%   Estimate PSD of JUST signal.
Snn = max(Smm_sCal - Svv, eps) / (sCal * sCal);
Snn = ClearEPS(Snn);  % Interpolate over the eps values.
%   Fixed whitener is 1./SQRT of power spectrum of JUST signal.
Htime = min( 1 ./ sqrt(Snn), MMaxGain);  % Assign zero phase.

% Iteratively compute Wiener filter, gain, and FIR filters.
Svv_tilda = Svv .* Htime .* Htime;  % PSD of noise after fixed whitener.
for i = 1:length(sCalVec)
  Hwiener = sCalVec(i)*sCalVec(i) ./ ( sCalVec(i)*sCalVec(i) + Svv_tilda );
  Htw = Htime .* Hwiener;  % Cascade Htime, Hwiener (both real).
  B(i,:) = fir2( Order, f, Htw );  % First pass, no gain scaling.
  AchievedH = abs(freqz(B(i,:), 1, NNfft/2+1));  % Achieved filter magnitude.
  Sinput = sCalVec(i)*sCalVec(i)*Snn + Svv;  % Input PSD at this EMG amplitude.
  d = sqrt( mean(Sinput) ./ mean(Sinput.*AchievedH.*AchievedH) );
  B(i,:) = B(i,:) * d;  % Proper gain scaling.
end

return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ClearEPS %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The minimum value in input Snn is found in all locations where an eps has
% been inserted.  This function replaces this value with a smooth
% interpolation, so that the ensuing inverse does not "blow up".
%   Originally writen by Punit Prakash, 2004.
function Snn = ClearEPS(Snn)
eps1 = find(Snn == min(Snn));   % find locations where minimum value occurs in noise
minval = min(Snn);              % store minimum value

if (eps1(1) == 1)
  first = find(Snn >= minval); % if first value in Snn is minval, replace with first non-minval value in Snn
  Snn(1) = first(1);
end
idx=length(Snn);                

if (eps1(end) == length(Snn))   % if last value in Snn is minval
  while (Snn(idx) == minval), idx =  idx-1; end          % find last "good value"
  num_eps = length(Snn) - idx;  
  for p = 1:(num_eps-1)
    Snn(idx+p) = Snn(idx) - (Snn(idx)/num_eps)*p;  % Interpolate between last good point and 0
  end
end

for z = 1:length(eps1)  
  x = eps1(z);    % Get index of location of minval
  a = x-1;        % Get index of point ahead
  b = x+1;        % Get index of point before
  if (a ~= 0 & b~=length(Snn) & b~=length(Snn)+1)
    if((Snn(a) ~= minval) & (Snn(b) ~= minval))
      Snn(x) = mean([Snn(a) Snn(b)]); % interpolate if Snn(a) and Snn(b) are "good" values
    else   
      if (b ~= length(Snn)-1)
        while (a~= 0 & Snn(a) == minval & a~=1), a = a - 1; end  % find last good value
        while (Snn(b) == minval)
          if (b == length(Snn) - 1)   % If last value is minval, then break
            break;
          else
            b = b + 1;               % loop to find next value
          end
        end
        temp = Snn(b) - Snn(a);          % interpolation    
        num_eps = b - a;
        for p = 1:(num_eps-1), Snn(a+p) = Snn(a) + (temp/num_eps)*p; end
      end
    end
  end
end

return;
