function M = e_fsim(EMGamp, varargin)
%E_FSIM  Functional stochastic simulation of the EMG waveform.
%   M = e_fsim(EMGamp, HspaceSpec, HtimeSpec, HnoiseSpec, Rstate)
%   simulates multiple channels of the EMG waveform with specified
%   time-varying EMG amplitude, spatial and temporal correlations
%   and additive noise.  EMGamp is the only required argument.  Unused
%   arguments on the tail may be omitted and the null vector "[]"
%   should be used as a placeholder.  The vector EMGamp gives the
%   desired EMG amplitude at each sample.  The length
%   of this vector is the length of the simulated sequences.  Output
%   argument M will hold the simulated EMG waveform sequences.
%   If EMGamp is a row vector, then each row of M holds
%   one EMG waveform channel and visa versa.
%
%   HspaceSpec specifies the spatial transformation matrix.  By default
%   this matrix is omitted and all channels are spatially uncorrelated.
%   Several formats exist for specifying HspaceSpec, as documented
%   in the User's Manual.  HtimeSpec specifies the temporal correlation
%   for each channel.  By default each channel is white.  Several
%   formats exist for specifying HtimeSpec, as documented in the
%   User's Manual.  HnoiseSpec specifies the additive Gaussian noise sources for
%   each channel.  The default additive noise intensities are zero.
%   Rstate is an optional initial state value for the random number generator.
%   If not specified or set to the null value, the generator is randomly
%   stated using a value generated from the date and time of day.
%
%   In addition to the above arguments, either of the strings
%   'Wgauss' or 'Wlaplace' may be appended to the argument list.
%   By default, or if 'Wgauss' is specified, the simulation is
%   driven by Gaussian distributed processes.  If 'Wlaplace' is
%   specified, the simulation is driven by Laplacian
%   distributed processes.
%
%   EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

% Copyright (c) Edward A. Clancy, 2007.
% This work is licensed under the Aladdin free public license.
% For copying permissions see license.txt.
% email: ted@wpi.edu

% 5 February 2007.

%%%%%%%%%%%%%%%%%%%%% Process Command Arguments %%%%%%%%%%%%%%%%%%%%%
% Deal first with the white noise density specification.
Wdensity = 'Wgauss';
NumArgs = nargin;  % First guess of number of numeric arguments.
VarArgs = length(varargin);
if VarArgs>0  &  isempty(varargin{VarArgs})  % Last optional arg is null.
  NumArgs = NumArgs - 1;
  VarArgs = VarArgs - 1;
end
if ( VarArgs~=0  &  isstr( varargin{VarArgs} ) )  % String density spec??
    Wdensity = varargin{VarArgs};
    NumArgs = NumArgs - 1;  % Adjust number of numeric arguments.
end

% Check number of input numeric arguments
if (NumArgs < 1  |  NumArgs > 5)
  error(['Bogus number of numeric arguments (' int2str(NumArgs) '), expected 1-5']);
end

% Get EMGamp and coerce it to be a COLUMN vector.
if (ndims(EMGamp) > 2), error('EMGamp is not a vector.'); end
[Erow, Ecol] = size(EMGamp);
if ( (Erow~=1) & (Ecol~=1) ), error('EMGamp must be a vector, not a matrix.'); end
if (min(EMGamp) < 0), error('EMGamp values must be non-negative.'); end
if (Erow==1), EMGamp = EMGamp'; end
N = length(EMGamp);  % Number of simulated EMG waveform samples per channel.

% Deal with HspaceSpec and determine number of simulated EMG channels (L).
Hspace = 1; L = 1;      % Defaults.
if (NumArgs >= 2)
  Hspace = TransMatrix( varargin{1}, 'HspaceSpec' );
  L = length(Hspace);
end

% HtimeSpec.
Htime  = ones(L,1); Torder = 0;  % Defaults.
if (NumArgs >= 3), [Htime, Torder]  = TimeMatrix( varargin{2}, 'HtimeSpec', L); end

% HnoiseSpec.
Hnoise = zeros(L,1); Norder = 0;  % Defaults.
if (NumArgs >= 4), [Hnoise, Norder] = TimeMatrix( varargin{3}, 'HnoiseSpec', L ); end

% Rstate.
Rstate = sum(100*clock);  % Default with a random state.
if (NumArgs>=5  &  isempty(varargin{4})==0), Rstate = varargin{4}; end
Wdensity  %% DEBUG

%%%%%%%%%%%%%%%%%%%% Run Model %%%%%%%%%%%%%%%%%%%%
% Note:  Must pad random sequences to account for startup times in FIR filters.

% Generate white noise, apply spatial transformation.  X has dimensions (N+Torder)xL.
randn('state', Rstate);  % Always initialize Gauss, since used for noise.
switch Wdensity
  case 'Wgauss'
    X = randn(N+Torder,L) * Hspace;
  case 'Wlaplace'
    rand('state', Rstate);
    X = ( (sqrt(2)/2) .* sign(rand(N+Torder,L)-0.5) .* log(rand(N+Torder,L)) ) * Hspace;
  otherwise, error('Bogus white noise density specification.');
end

% Temporally shape, modulate by EMG amplitude (R has dimensions NxL).
R = zeros(N,L);  % Pre-allocate.
for i=1:L
  Temp = filter( Htime(i,:), 1, X(:,i) );  % Filtered sequence, padded at front.
  R(:,i) = EMGamp .* Temp(Torder+1:N+Torder);
end
clear X Temp;

% Additive noise.  VV (always Gaussian) has padded dimensions (N+Norder)xL).  V is NxL.
VV = randn(N+Norder,L);
for i=1:L
  Temp = filter( Hnoise(i,:), 1, VV(:,i) );  % Filtered sequence, padded at front.
  V(:,i) = Temp(Norder+1:N+Norder);
end
clear VV Temp

% Add noise to signal.
M = V + R;
clear V R

% Return vector of desired dimension.
if (Erow==1), M = M'; end
return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%% Function TransMatrix() %%%%%%%%%%%%%%%%%%%%%%%
function MatOut = TransMatrix(Spec, ArgName)
% Construct a square spatial transformation matrix MatOut based
% on the specification contained in Spec.  Each column of MatOut
% stores the weights to generate one output channel.  Description of the
% specification formats is contained in the User's Manual.  ArgName
% is a string with the name of the specification argument (used
% in error messages).

% First, deal with the null vector default.
if isempty(Spec), MatOut = 1; return; end
if (length(Spec)==1), error([ArgName ' must not have length one.']); end

% Now, look for the remaining cases.
if (ndims(Spec) > 2), error([ArgName ' must have <=2 dimensions.']); end
[Row, Col] = size(Spec);
if (Row == Col)  % One of the square matrix formats.
  if isnan( Spec(1,1) )  % Correlation coefficient matrix is specified.
    Spec(1,1) = 1;  % Remove NaN.
    for i=2:length(Spec)
      if (Spec(i,i)~=1), error(['Corr coef matrix ' ArgName ' must have ones on diagonal.']); end
    end
    if (any(any(Spec-Spec'))~=0), error(['Corr coef matrix ' ArgName ' must be symmetric.']); end
    if (min(min(Spec))<-1 | max(max(Spec))>1), error(['Bogus corr coef in ' ArgName '.']); end
    % Create MatOut from desired correlation coefficients.
    MatOut = TransFromCorr(Spec);
  else  % Transformation matrix is specified.
    MatOut = Spec;
  end
else  % Must be the two-element vector format.
  if (Row*Col ~= 2), error(['Bogus size for ' ArgName '.']); end
  if (Spec(2)<-1 | Spec(2)>1), error(['Bogus corr coef in ' ArgName '.']); end
  CorrCoefMat = Spec(2) * ones(Spec(1),Spec(1));
  for i=1:Spec(1), CorrCoefMat(i,i) = 1; end
  % Create MatOut from desired correlation coefficients.
  MatOut = TransFromCorr(CorrCoefMat);
end

return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Function TransFromCorr() %%%%%%%%%%%%%%%%%%%%%%
function MatOut = TransFromCorr(MatIn)
% Construct a square spatial transformation matrix MatOut based on
% a description of the desired output signal correlation matrix
% MatIn.  The mathematics to do so is described in E.A. Clancy's
% Ph.D. Thesis (pp. 374-375, 1991).  (Note:  In the thesis, the shaping
% matrix is multiplied on the left, in this script the shaping
% matrix is multiplied on the right.  Thus, the shaping matrix in this
% script is the transpose of that found in the thesis math.)

% Get the orthonormal eigenvectors, and associated eigenvalues
%   of the desired output correlation matrix.
[V, D] = eig(MatIn);

% Each column of V is an eigenvector.  Multiply V by a matrix made
%   by square-rooting the magnitude portion of D, preserving the sign.
MatOut = V * ( sqrt(abs(D)).*sign(D) );
MatOut = MatOut';  % Correct the orientation.

return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% TimeMatrix() %%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [MatOut, Order] = TimeMatrix(Spec, ArgName, L)
% Construct a temporal correlating matrix MatOut based
% on the specification contained in Spec.  Each row of MatOut stores
% the FIR filter coefficients used to correlate one channel.
% Description of the specification formats is contained in the User's
% Manual.  ArgName is a string with the name of the specification
% argument (used in error messages).  L is the number of EMG channels.
% Output Order is the order of the FIR filter.

% First, deal with the null vector default.
if isempty(Spec), MatOut = ones(L,1); Order = 0; return; end

% Now, look for the remaining cases.
if (ndims(Spec) > 2), error([ArgName ' must have <=2 dimensions.']); end
[Row, Col] = size(Spec);
if (Row==L), MatOut = Spec; Order = Col-1;  % Matrix format.
elseif (Row==2*L+1 & Col==1)  % Vector format.
  Order = Spec(1);
  if (Order<1), error(['Bogus order argument in ' ArgName '.']); end
  if ( any(Spec(3:2:2*L+1) < 0)  |  any(Spec(3:2:2*L+1) > 1) )
    error(['Bogus frequency argument in ' ArgName '.']);
  end
  MatOut = zeros(L, Order+1);  % Pre-allocate.
  for i=1:L, MatOut(i,:) = Spec(2*i) * fir1(Order, Spec(2*i+1)); end
else
  error(['Bogus dimensions (' int2str(Row) 'x' int2str(Col) ') for ' ArgName '.']);
end

return
