function Uran = e_uband(N, Wn, varargin)
%E_UBAND Create uniform-distributed, band-limited, random process.
%   Uran = e_uband(N, Wn, Rstate, Method) creates an N-length column vector
%   of uniform-distributed, band-limited, random numbers.  The desired power
%   spectrum of the simulated sequence is flat from DC to the cutoff
%   frequecy Wn, and zero thereafter.  Wn is a number between 0 and 1
%   (excluding 0), where 1 corresponds to half the sampling frequency (the
%   Nyquist frequency).  Rstate is an optional initial state value for the
%   random number generator.  If not specified, the generator is randomly
%   stated using a value generated from the date and time of day.  Method is
%   an optional string argument which must equal either 'Interp' or 'Filter'.
%   (Note, Method may be present even if Rstate is ommitted.)
%
%   In general, creating correlated non-Gaussian sequences is a difficult
%   problem with no unique solution.  The two solution methods implemented
%   here both have limitations.  Neither provides a first-order density
%   which is precisely Uniform nor a bandwidth which precisely meets the
%   desired specification.  However, in many cases the methods generate
%   sequences which are acceptatable for the problem under study.  The user
%   should evaluate the characteristics of the resulting random sequence to
%   determine if it is acceptable for the particular application to which
%   it will be applied.
%
%   By default, or if Method = 'Interp', the sequence is generated by a
%   three-stage process.  First, N1=N*Wn INDEPENDENT, GAUSSIAN-distributed,
%   random deviates are created.  These represent the total number of
%   statistically independent samples in the complete sequence.  Second,
%   the N1 samples are interpolated up to N samples using spline
%   interpolation.  This stage preserves the desired number of degrees of
%   freedom in the sequence and creates the correct sequence length.  The
%   sequence is nearly (but not precisely) Gaussian at this point, with
%   the desired bandwidth.  Third, the sequence is transformed by the
%   Gaussian cummulative density function.  For exactly (independent)
%   Gaussian inputs, the resulting signal would be uniform.  Although the
%   transformation does not exactly preserve the signal correlation,
%   this algorithm has worked well when Wn is small (i.e. less than 0.05).
%
%   If Method = 'Filter', the sequence is generated by a different
%   three-stage process.  First, N INDEPENDENT, GAUSSIAN-distributed,
%   random deviates are created.  Second, the sequence is filtered by
%   a tenth-order Butterworth filter (cutoff frequency set at Wn).  The
%   filtered sequence is Gaussian distributed with a bandwidth
%   approximated by the tenth-order filter.  Third, the sequence is
%   transformed by the Gaussian cummulative density function.  The
%   resulting signal would be uniform IF the Gaussian data were
%   independent.  The user should be aware of some
%   limitations of this method.  First, the cummulative density
%   transformation does not strictly preserve the signal correlation.
%   Thus, each application must verify that the resulting bandwidth
%   is close enough to the desired specification.  Second, the
%   Butterworth filter has a start-up transient that depends on Wn.
%   This transient typically dies out exponentially.  Thus, 100/Wn extra
%   random deviates are processed and the first 100/Wn (representing
%   the startup transient) are discarded.  User's should verify that
%   this length is sufficient to eliminate the transient.  Third, this
%   method seems to fail for very small Wn (i.e. less than 0.05).
%   Fourth, the lower the value of Wn, the poorer the resulting density
%   approximates the Uniform density.  (The density looks more Gaussian
%   as Wn decreases.)  Values of Wn above 0.9 seem to work well.
%
%   EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

% Copyright (c) Edward A. Clancy, 2007.
% This work is licensed under the Aladdin free public license.
% For copying permissions see license.txt.
% email: ted@wpi.edu

% 5 February 2007.

%%%%%%%%%%%%%%%%%%%%% Check Command Arguments %%%%%%%%%%%%%%%%%%%%%

% Deal first with the method specification.
Method = 'Interp';
NumArgs = nargin;  % First guess of number of numeric arguments.
if ( length(varargin)~=0  &  isstr( varargin{length(varargin)} ) )
    Method = varargin{length(varargin)};
    NumArgs = NumArgs - 1;  % Adjust number of numeric arguments.
end
% Check for a null method specification.
if NumArgs==4 & isempty(varargin{2}), NumArgs=3; end

% Check number of input numeric arguments
if (NumArgs < 2  |  NumArgs > 3)
  error(['Bogus number of numeric arguments (' int2str(NumArgs) '), expected 2-4.']);
end

if (length(N) ~= 1), error('Sequence length "N" is not a scalor.'); end
if (N<=0), error('Sequence length "N" must be positive.'); end
if (length(Wn) ~= 1), error('"Wn" is not a scalor.'); end
if (Wn<=0 | Wn>1), error('"Wn" is out of range.'); end

% Rstate.
Rstate = sum(100*clock);  % Default with a random state.
if (NumArgs>=3  &  isempty(varargin{1})==0), Rstate = varargin{1}; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Compute %%%%%%%%%%%%%%%%%%%%%%%%%%%%

randn('state', Rstate);  % Initialize Gaussian random numbers.
switch Method
  case 'Interp'
    % Generate INDEPENDENT Gaussian random deviates (pad at both ends).
    Pad = 5;                         % Pad each end with independent deviates.
    L = (2*Pad) + ceil(N*Wn);        % Desired number of independent deviates.
    Range = 1-Pad/Wn:1/Wn:N+Pad/Wn;  % Their locations (padding set out of 1:N).
    L = length(Range);               % Actual length (accounts for round-off).
    S1 = randn(L,1);
    % Spline interpolate to N samples, discarding beginning and end.
    S2 = spline(Range, S1, 1:N)';
    clear S1;
    % Transform to uniform density (0--1).
    Uran = normcdf(S2, 0, 1);
  case 'Filter'
    % Generate Gaussian random deviates (with padding).
    if (Wn==1), error('Wn can not equal 1 when Method = "Filter".'); end
    Pad = ceil(100/Wn);
    S1 = randn(N+Pad, 1);
    % Filter.
    [b,a] = butter(10, Wn);
    S2 = filter(b, a, S1);
    clear S1;
    % Transform to uniform density (0--1), discarding beginning pad.
    Uran = normcdf(S2(1+Pad:N+Pad), 0, 1);
  otherwise, error('Bogus method specification.');
end
