function EMGwhite = e_whiten(EMGin, Bwhite, sCalVec, EMGampGuess, varargin)
%E_WHITEN Adaptively whiten a signal based on the adaptation designed in e_cal_wh().
%
% EMGwhite = e_whiten(EMGin, Bwhite, sCalVec, EMGampGuess [, Causality] [, Edges] [, Sum])
%
% EMGin: Input EMG waveform (vector).
% Bwhite:Each row contains the moving average coefficients of the
%        complete adaptive whitening filter corresponding to the
%        EMG amplitude found in the corresponding element of sCalVec.
% sCalVec: EMG amplitude levels.  Levels must be in equal increments,
%        starting just above zero and going to the maximum expected
%        level. (vector - length equal to the number of rows in Bwhite).
% EMGampGuess: Input "first guess" of the EMG amplitude.
% Causality: Optional argument specifying the causality as 'Noncausal',
%        (default), 'Causal' or h.  Null results in use of default.
% Edges: Optional string argument that tells how to treat filtering at
%        the tails.  Must be "EdgesFixed", "EdgesZero" or "EdgesNaN".
% Sum:   Optional string argument that tells how to perform the filter
%        summing.  Must be "SumFast" or "SumFull".
%
% EMGwhite:  Whitened signal (vector).
%
% EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

% Copyright (c) Edward A. Clancy, 2007.
% This work is licensed under the Aladdin free public license.
% For copying permissions see license.txt.
% email: ted@wpi.edu

% 5 February 2007.

% Bcol:   Number of columns in Bwhite.
% Bindex: Column vector of length Elen.  Each element gives the row
%           in Bwhite used for that corresponding sample.
% Brow:   Number of rows in Bwhite.
% Gcol:   Number of columns in EMGampGuess.
% Grow:   Number of rows in EMGampGuess.
% Scol:   Number of columns in sCalVec.
% Sincr:  Increment value used in sCalVec.
% Slen:   Length of sCalVec.
% Srow:   Number of rows in sCalVec.
% Ecol:   Number of columns in EMGin (as input).
% Elen:   Length of EMGin (as input).
% Erow:   Number of rows in EMGin (as input).


%%%%%%%%%%%%%%%%%%%%% Check Command Arguments %%%%%%%%%%%%%%%%%%%%%

if (nargin<4 | nargin>7)
  error(['Bogus number of arguments (' int2str(nargin) '), expected 4-6.']);
end

% Check EMGin.
if ischar(EMGin), error('Argument 1 must be numeric.'); end
if ndims(EMGin)>2, error('Argument 1 must be a vector.'); end
[Erow, Ecol] = size(EMGin);
if Erow*Ecol ~= length(EMGin), error('Argument 1 must be a vector'); end
if Erow~=1, EMGin = EMGin'; end  % Coerce to row vector.
Elen = length(EMGin);

% Check Bwhite.
if ischar(Bwhite), error('Argument 2 must be numeric.'); end
if ndims(Bwhite)~=2, error('Argument 2 must be a matrix.'); end
[Brow, Bcol] = size(Bwhite);

% Check sCalVec.
if ischar(sCalVec), error('Argument 3 must be numeric.'); end
if ndims(sCalVec)>2, error('Argument 3 must be a vector.'); end
[Srow, Scol] = size(sCalVec);
Slen = length(sCalVec);
if Srow*Scol ~= Slen, error('Argument 3 must be a vector.'); end
if Slen~=Brow, error('Length of Arg 3 must match number of rows in Arg2.'); end
if Slen<=1, Sincr = 1.0; else Sincr = sCalVec(2) - sCalVec(1); end
if Sincr<0, error('Second value in Argument 3 < first value.'); end
sCheck = abs( sCalVec(1) + ((0:Slen-1) * Sincr) - sCalVec );
if any( sCheck > Sincr/1000 ), error('Argument 3 not equal increments.'); end

% Check EMGampGuess.
if ischar(EMGampGuess), error('Argument 4 must be numeric.'); end
if ndims(EMGampGuess)>2, error('Argument 4 must be a vector.'); end
[Grow, Gcol] = size(EMGampGuess);
if Grow*Gcol ~= length(EMGampGuess), error('Argument 4 must be a vector.'); end
if length(EMGampGuess)~=Elen, error('Args 1 and 4 must have same length.'); end
if min(EMGampGuess)<0, error('All elements in Arg 4 must be non-negative.'); end

% Optional arguments.
%   Set defaults and flags.
h = fix(Bcol/2);  % Default for noncausal processing.
Edges = 'EdgesFixed';
Sum   = 'SumFast';
FlagCausality = 0;  % Zero denotes option in this catagory has not been
FlagEdges     = 0;  %   specified.  One denotes otherwise.
FlagSum       = 0;
%   Process the options.
for i = 1:length(varargin)
  if ischar( varargin{i} )
    switch varargin{i}
      case 'Noncausal'
        if (FlagCausality > 0 ), error('Repeated causality selection.'); end
          FlagCausality = 1;  % This is the default - no action taken.
      case 'Causal'
        if (FlagCausality > 0 ), error('Repeated causality selection.'); end
        FlagCausality = 1;
        h = 0;
      case {'EdgesFixed', 'EdgesZero', 'EdgesNaN'}
        if (FlagEdges > 0 ), error('Repeated edge selection.'); end
        FlagEdges = 1;
        Edges = varargin{i};
      case {'SumFast', 'SumFull'}
        if (FlagSum > 0 ), error('Repeated sum selection.'); end
        FlagSum = 1;
        Sum = varargin{i};
      otherwise
        error(['Bogus option "' varargin{i} '".']);
    end  % switch.
  elseif isempty( varargin{i} )  % Do nothing for null arguments.
  else  % Must be h.
    if (FlagCausality > 0 ), error('Repeated causality selection.'); end
    FlagCausality = 1;
    if length(varargin{i})>1, error('Numeric causality spec must be scalor.'); end
    if length(varargin{i})==1  % Else, leave default for null value.
      if varargin{i}<0, error('Numeric causality spec must be >= 0.'); end
      h = round( varargin{i} );
    end
  end  % Must be h.
end  % for loop.

if h==fix(Bcol/2)  &  2*round(Bcol/2)==Bcol
  error('Arg 2 must have odd number of columns for noncausal processing.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Compute %%%%%%%%%%%%%%%%%%%%%%%%%

% Determine which filter to use at each time instant.
EMGampGuess = max( EMGampGuess, sCalVec(1) );     % Trim values to range min.
EMGampGuess = min( EMGampGuess, sCalVec(Slen) );  % Trim values to range max.
Bindex = round(  (EMGampGuess - sCalVec(1)) ./ Sincr  ) + 1;
EMGwhite = zeros(size(EMGin));  % Pre-allocate.
Head = Bcol - 1 - h;  % Index of last  leading  edge.
Tail = Elen + 1 - h;  % Index of first trailing edge.

switch Sum
  case 'SumFast'
    if exist('e__whi')~=3, error('MEX file not found for SumFast option.'); end
    e__whi(EMGwhite, EMGin, Bwhite, Bindex, h);
  case 'SumFull'
    % Process leading edge.
    for i=1:Head
      EMGwhite(i) = sum( Bwhite(Bindex(i),Bcol-i-h+1:Bcol) .* EMGin(1:i+h) );
    end
    % Process body.
    for i=Head+1:Tail-1
      EMGwhite(i) = sum( Bwhite(Bindex(i),:) .* EMGin(i-Bcol+1+h:i+h) );
    end
    % Process trailing edge.
    for i=Tail:Elen
      EMGwhite(i) = sum( Bwhite(Bindex(i),1:Elen-i+Bcol-h) .* EMGin(i-Bcol+1+h:Elen) );
    end
  otherwise
    error('Internal "Sum" value error.');
end

% Satisfy edge specification.
switch Edges
  case 'EdgesFixed' % Do nothing.
  case 'EdgesNaN',  EMGwhite(1:Head) = NaN; EMGwhite(Tail:Elen) = NaN;
  case 'EdgesZero', EMGwhite(1:Head) = 0;   EMGwhite(Tail:Elen) = 0;
end

% Coerce to column vector, if necessary.
if Erow~=1, EMGwhite = EMGwhite'; end
