function N1 = e_win1(EMGamp, EMGderiv1, f, g, VarWeight)
%E_WIN1 Calculate adaptive smoothing window lengths:  first derivative, causal technique.
%   N1 = e_win1(EMGamp, EMGderiv1, f, g, VarWeight) calculates adaptive
%   smoothing window lengths using the first derivative, causal technique.
%   The window length depends on the existing estimate of the EMG amplitude
%   and its first derivative, the sampling rate f and the modifier g (which
%   depends on the statistical bandwith of the EMG waveform, the number of
%   EMG channels and the detector type).  The tradeoff between error in the
%   variance component of the EMG amplitude estimate and the squared-bias
%   component can be altered with the optional argument VarWeight.
%
%   EMGamp and EMGderiv1 are typically vectors (their dimensions must match) in
%   which each element contains the existing estimate of the EMG amplitude and its
%   corresponding first derivative, respectively.  The sampling rate f is a scalor
%   in units of Hertz.  The modifier g is a scalor in units of Hertz.  If supplied,
%   optional argument VarWeight is a scalor which weights the relative error
%   in the variance component of the EMG amplitude estimate error versus the
%   squared-bias component.  The default value of VarWeight is 1.0, giving
%   equal weight to the two components.  Relative changes in VarWeight alter
%   the relative weighting.  Values greater than one give more emphasis to
%   variance error while values less than one give more emphasis to
%   squared-bias error.  Negative weight values and values of zero or infinity
%   are not permitted.
%
%   Function e_win1 returns the vector of window lengths N1, which has the
%   same dimensions as EMGamp and EMGderiv1.
%
%   Algorithm:  N1 = f * (VarWeight/g)**(1/3) * (EMGamp**2/EMGderiv1**2)**(1/3).
%
%   EMG Amplitude Estimation Toolbox - Ted Clancy - WPI

% Copyright (c) Edward A. Clancy, 2007.
% This work is licensed under the Aladdin free public license.
% For copying permissions see license.txt.
% email: ted@wpi.edu

% 5 February 2007.

%%%%%%%%%%%%%%%%%%%%% Check Command Arguments %%%%%%%%%%%%%%%%%%%%%

% Check number of input arguments
if (nargin < 4  |  nargin > 5)
  error(['Bogus number of arguments (' int2str(nargin) '), expected 4-5.']);
end

% Check input values, as appropriate.
Test = sum( size(EMGamp) - size(EMGderiv1) );
if (Test ~= 0), error('"EMGamp" and "EMGderiv1" dimensions must match.'); end
if (f<=0), error('Sampling rate "f" must be positive.'); end
if (length(f) ~= 1), error('Sampling rate "f" is not a scalor.'); end
if (g<=0), error('Modifier "g" must be positive.'); end
if (length(g) ~= 1), error('Mofifier "g" is not a scalor.'); end
if (nargin == 5  &  isempty(VarWeight) == 0)
  if (VarWeight<=0), error('Weight factor "VarWeight" must be positive.'); end
  if (length(VarWeight) ~= 1), error('Weight factor "VarWeight" is not a scalor.'); end
else
  VarWeight = 1.0;  % Assign the default.
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Compute %%%%%%%%%%%%%%%%%%%%%%%%%

N1 = f * ( (VarWeight/g).^(1/3) ) * ( (EMGamp.^2./EMGderiv1.^2).^(1/3) );
